@@ -1,5 +1,6 @@
 from decimal import Decimal
 
+from sqlalchemy import column
 from sqlalchemy import exc
 from sqlalchemy import ForeignKey
 from sqlalchemy import func
@@ -18,7 +19,9 @@ from sqlalchemy.orm import declarative_base
 from sqlalchemy.orm import relationship
 from sqlalchemy.orm import Session
 from sqlalchemy.orm import synonym
+from sqlalchemy.sql import coercions
 from sqlalchemy.sql import operators
+from sqlalchemy.sql import roles
 from sqlalchemy.sql import update
 from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import AssertsCompiledSQL
@@ -26,6 +29,7 @@ from sqlalchemy.testing import eq_
 from sqlalchemy.testing import fixtures
 from sqlalchemy.testing import is_
 from sqlalchemy.testing import is_false
+from sqlalchemy.testing import is_not
 from sqlalchemy.testing.fixtures import fixture_session
 from sqlalchemy.testing.schema import Column
 
@@ -33,7 +37,7 @@ from sqlalchemy.testing.schema import Column
 class PropertyComparatorTest(fixtures.TestBase, AssertsCompiledSQL):
     __dialect__ = "default"
 
-    def _fixture(self):
+    def _fixture(self, use_inplace=False, use_classmethod=False):
         Base = declarative_base()
 
         class UCComparator(hybrid.Comparator):
@@ -54,9 +58,33 @@ class PropertyComparatorTest(fixtures.TestBase, AssertsCompiledSQL):
                 "This is a docstring"
                 return self._value - 5
 
-            @value.comparator
-            def value(cls):
-                return UCComparator(cls._value)
+            if use_classmethod:
+                if use_inplace:
+
+                    @value.inplace.comparator
+                    @classmethod
+                    def _value_comparator(cls):
+                        return UCComparator(cls._value)
+
+                else:
+
+                    @value.comparator
+                    @classmethod
+                    def value(cls):
+                        return UCComparator(cls._value)
+
+            else:
+                if use_inplace:
+
+                    @value.inplace.comparator
+                    def _value_comparator(cls):
+                        return UCComparator(cls._value)
+
+                else:
+
+                    @value.comparator
+                    def value(cls):
+                        return UCComparator(cls._value)
 
             @value.setter
             def value(self, v):
@@ -70,31 +98,51 @@ class PropertyComparatorTest(fixtures.TestBase, AssertsCompiledSQL):
         eq_(a1._value, 10)
         eq_(a1.value, 5)
 
-    def test_value(self):
-        A = self._fixture()
+    @testing.variation("use_inplace", [True, False])
+    @testing.variation("use_classmethod", [True, False])
+    def test_value(self, use_inplace, use_classmethod):
+        A = self._fixture(
+            use_inplace=use_inplace, use_classmethod=use_classmethod
+        )
         eq_(str(A.value == 5), "upper(a.value) = upper(:upper_1)")
 
-    def test_aliased_value(self):
-        A = self._fixture()
+    @testing.variation("use_inplace", [True, False])
+    @testing.variation("use_classmethod", [True, False])
+    def test_aliased_value(self, use_inplace, use_classmethod):
+        A = self._fixture(
+            use_inplace=use_inplace, use_classmethod=use_classmethod
+        )
         eq_(str(aliased(A).value == 5), "upper(a_1.value) = upper(:upper_1)")
 
-    def test_query(self):
-        A = self._fixture()
+    @testing.variation("use_inplace", [True, False])
+    @testing.variation("use_classmethod", [True, False])
+    def test_query(self, use_inplace, use_classmethod):
+        A = self._fixture(
+            use_inplace=use_inplace, use_classmethod=use_classmethod
+        )
         sess = fixture_session()
         self.assert_compile(
             sess.query(A.value), "SELECT a.value AS a_value FROM a"
         )
 
-    def test_aliased_query(self):
-        A = self._fixture()
+    @testing.variation("use_inplace", [True, False])
+    @testing.variation("use_classmethod", [True, False])
+    def test_aliased_query(self, use_inplace, use_classmethod):
+        A = self._fixture(
+            use_inplace=use_inplace, use_classmethod=use_classmethod
+        )
         sess = fixture_session()
         self.assert_compile(
             sess.query(aliased(A).value),
             "SELECT a_1.value AS a_1_value FROM a AS a_1",
         )
 
-    def test_aliased_filter(self):
-        A = self._fixture()
+    @testing.variation("use_inplace", [True, False])
+    @testing.variation("use_classmethod", [True, False])
+    def test_aliased_filter(self, use_inplace, use_classmethod):
+        A = self._fixture(
+            use_inplace=use_inplace, use_classmethod=use_classmethod
+        )
         sess = fixture_session()
         self.assert_compile(
             sess.query(aliased(A)).filter_by(value="foo"),
@@ -189,7 +237,8 @@ class PropertyComparatorTest(fixtures.TestBase, AssertsCompiledSQL):
 class PropertyExpressionTest(fixtures.TestBase, AssertsCompiledSQL):
     __dialect__ = "default"
 
-    def _fixture(self):
+    def _fixture(self, use_inplace=False, use_classmethod=False):
+        use_inplace, use_classmethod = bool(use_inplace), bool(use_classmethod)
         Base = declarative_base()
 
         class A(Base):
@@ -202,15 +251,42 @@ class PropertyExpressionTest(fixtures.TestBase, AssertsCompiledSQL):
                 "This is an instance-level docstring"
                 return int(self._value) - 5
 
-            @value.expression
-            def value(cls):
-                "This is a class-level docstring"
-                return func.foo(cls._value) + cls.bar_value
-
             @value.setter
             def value(self, v):
                 self._value = v + 5
 
+            if use_classmethod:
+                if use_inplace:
+
+                    @value.inplace.expression
+                    @classmethod
+                    def _value_expr(cls):
+                        "This is a class-level docstring"
+                        return func.foo(cls._value) + cls.bar_value
+
+                else:
+
+                    @value.expression
+                    @classmethod
+                    def value(cls):
+                        "This is a class-level docstring"
+                        return func.foo(cls._value) + cls.bar_value
+
+            else:
+                if use_inplace:
+
+                    @value.inplace.expression
+                    def _value_expr(cls):
+                        "This is a class-level docstring"
+                        return func.foo(cls._value) + cls.bar_value
+
+                else:
+
+                    @value.expression
+                    def value(cls):
+                        "This is a class-level docstring"
+                        return func.foo(cls._value) + cls.bar_value
+
             @hybrid.hybrid_property
             def bar_value(cls):
                 return func.bar(cls._value)
@@ -412,22 +488,34 @@ class PropertyExpressionTest(fixtures.TestBase, AssertsCompiledSQL):
             "a.lastname AS name FROM a) AS anon_1",
         )
 
-    def test_info(self):
-        A = self._fixture()
+    @testing.variation("use_inplace", [True, False])
+    @testing.variation("use_classmethod", [True, False])
+    def test_info(self, use_inplace, use_classmethod):
+        A = self._fixture(
+            use_inplace=use_inplace, use_classmethod=use_classmethod
+        )
         inspect(A).all_orm_descriptors.value.info["some key"] = "some value"
         eq_(
             inspect(A).all_orm_descriptors.value.info,
             {"some key": "some value"},
         )
 
-    def test_set_get(self):
-        A = self._fixture()
+    @testing.variation("use_inplace", [True, False])
+    @testing.variation("use_classmethod", [True, False])
+    def test_set_get(self, use_inplace, use_classmethod):
+        A = self._fixture(
+            use_inplace=use_inplace, use_classmethod=use_classmethod
+        )
         a1 = A(value=5)
         eq_(a1._value, 10)
         eq_(a1.value, 5)
 
-    def test_expression(self):
-        A = self._fixture()
+    @testing.variation("use_inplace", [True, False])
+    @testing.variation("use_classmethod", [True, False])
+    def test_expression(self, use_inplace, use_classmethod):
+        A = self._fixture(
+            use_inplace=use_inplace, use_classmethod=use_classmethod
+        )
         self.assert_compile(
             A.value.__clause_element__(), "foo(a.value) + bar(a.value)"
         )
@@ -435,8 +523,6 @@ class PropertyExpressionTest(fixtures.TestBase, AssertsCompiledSQL):
     def test_expression_isnt_clause_element(self):
         A = self._wrong_expr_fixture()
 
-        from sqlalchemy.sql import coercions, roles
-
         with testing.expect_raises_message(
             exc.InvalidRequestError,
             'When interpreting attribute "A.value" as a SQL expression, '
@@ -455,15 +541,23 @@ class PropertyExpressionTest(fixtures.TestBase, AssertsCompiledSQL):
             "AND foo(a.value) + bar(a.value) = :param_1)",
         )
 
-    def test_aliased_expression(self):
-        A = self._fixture()
+    @testing.variation("use_inplace", [True, False])
+    @testing.variation("use_classmethod", [True, False])
+    def test_aliased_expression(self, use_inplace, use_classmethod):
+        A = self._fixture(
+            use_inplace=use_inplace, use_classmethod=use_classmethod
+        )
         self.assert_compile(
             aliased(A).value.__clause_element__(),
             "foo(a_1.value) + bar(a_1.value)",
         )
 
-    def test_query(self):
-        A = self._fixture()
+    @testing.variation("use_inplace", [True, False])
+    @testing.variation("use_classmethod", [True, False])
+    def test_query(self, use_inplace, use_classmethod):
+        A = self._fixture(
+            use_inplace=use_inplace, use_classmethod=use_classmethod
+        )
         sess = fixture_session()
         self.assert_compile(
             sess.query(A).filter_by(value="foo"),
@@ -471,8 +565,12 @@ class PropertyExpressionTest(fixtures.TestBase, AssertsCompiledSQL):
             "FROM a WHERE foo(a.value) + bar(a.value) = :param_1",
         )
 
-    def test_aliased_query(self):
-        A = self._fixture()
+    @testing.variation("use_inplace", [True, False])
+    @testing.variation("use_classmethod", [True, False])
+    def test_aliased_query(self, use_inplace, use_classmethod):
+        A = self._fixture(
+            use_inplace=use_inplace, use_classmethod=use_classmethod
+        )
         sess = fixture_session()
         self.assert_compile(
             sess.query(aliased(A)).filter_by(value="foo"),
@@ -480,8 +578,12 @@ class PropertyExpressionTest(fixtures.TestBase, AssertsCompiledSQL):
             "FROM a AS a_1 WHERE foo(a_1.value) + bar(a_1.value) = :param_1",
         )
 
-    def test_docstring(self):
-        A = self._fixture()
+    @testing.variation("use_inplace", [True, False])
+    @testing.variation("use_classmethod", [True, False])
+    def test_docstring(self, use_inplace, use_classmethod):
+        A = self._fixture(
+            use_inplace=use_inplace, use_classmethod=use_classmethod
+        )
         eq_(A.value.__doc__, "This is a class-level docstring")
 
         # no docstring here since we get a literal
@@ -854,6 +956,84 @@ class SynonymOfPropertyTest(fixtures.TestBase, AssertsCompiledSQL):
         )
 
 
+class InplaceCreationTest(fixtures.TestBase, AssertsCompiledSQL):
+    """test 'inplace' definitions added for 2.0 to assist with typing
+    limitations.
+
+    """
+
+    __dialect__ = "default"
+
+    def test_property_integration(self, decl_base):
+        class Person(decl_base):
+            __tablename__ = "person"
+            id = Column(Integer, primary_key=True)
+            _name = Column(String)
+
+            @hybrid.hybrid_property
+            def name(self):
+                return self._name
+
+            @name.inplace.setter
+            def _name_setter(self, value):
+                self._name = value.title()
+
+            @name.inplace.expression
+            def _name_expression(self):
+                return func.concat("Hello", self._name)
+
+        p1 = Person(_name="name")
+        eq_(p1.name, "name")
+        p1.name = "new name"
+        eq_(p1.name, "New Name")
+
+        self.assert_compile(Person.name, "concat(:concat_1, person._name)")
+
+    def test_method_integration(self, decl_base):
+        class A(decl_base):
+            __tablename__ = "a"
+            id = Column(Integer, primary_key=True)
+            _value = Column("value", String)
+
+            @hybrid.hybrid_method
+            def value(self, x):
+                return int(self._value) + x
+
+            @value.inplace.expression
+            def _value_expression(cls, value):
+                return func.foo(cls._value, value) + value
+
+        a1 = A(_value="10")
+        eq_(a1.value(5), 15)
+
+        self.assert_compile(A.value(column("q")), "foo(a.value, q) + q")
+
+    def test_property_unit(self):
+        def one():
+            pass
+
+        def two():
+            pass
+
+        def three():
+            pass
+
+        prop = hybrid.hybrid_property(one)
+
+        prop2 = prop.inplace.expression(two)
+
+        prop3 = prop.inplace.setter(three)
+
+        is_(prop, prop2)
+        is_(prop, prop3)
+
+        def four():
+            pass
+
+        prop4 = prop.setter(four)
+        is_not(prop, prop4)
+
+
 class MethodExpressionTest(fixtures.TestBase, AssertsCompiledSQL):
     __dialect__ = "default"
 
